// ----------------------------------------------------------------------
// Touch.cpp - touch selected files (Tracker)
//
// By Chris Herborth (arcanedragon@home.com)
//
// Copyright © 1999 Arcane Dragon Software.  All rights reserved.
//
// See the accompanying file LICENSE for terms of use.  If, for
// some reason, the LICENSE file is missing, the Arcane Dragon Software
// Source License may also be found at:
//
// http://home.beoscentral.com/chrish/ads/LICENSE.html
//
// June 28, 1999 - initial version
//
// Note that the process_refs() function is at the bottom.

// ----------------------------------------------------------------------
// Standard headers
#include <storage/Entry.h>
#include <interface/Alert.h>
#include <app/Application.h>
#include <support/TypeConstants.h>
#include <app/Roster.h>

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

// ----------------------------------------------------------------------
// Exported functions
#include <add-ons/tracker/TrackerAddOn.h>

// ----------------------------------------------------------------------
// Local prototypes

void DoError( const char *message );

// ----------------------------------------------------------------------
// Gimpy error handler.

void DoError( const char *message )
{
	try {
		BAlert *alert = new BAlert( "Touch Error",
									message,
									"Dang", NULL, NULL,
									B_WIDTH_AS_USUAL,
									B_STOP_ALERT );
		(void)alert->Go();
	} catch ( bad_alloc &ex ) {
		printf( "aieee... - %s\n", message );
	}
}

// ----------------------------------------------------------------------
// main() and BApplication to make this a standalone program as well
// as a Tracker add-on.

class TouchApplication : public BApplication {
public:
	TouchApplication( void )
		: BApplication( "application/x-vnd.ads.touch" ),
		  did_some_work( false )
	{
		;	// do nothing
	}

	virtual ~TouchApplication( void )
	{
		;	// do nothing
	}

	virtual void RefsReceived( BMessage *msg )
	{
		entry_ref ref;
		BEntry ent( "/boot" );
		(void)ent.GetRef( &ref );
		process_refs( ref, msg, this );
	}

	virtual void ReadyToRun( void )
	{
		if( !did_some_work ) {
			// TODO: BAlert error - select files/folders first? nah, do nothing
			BAlert *info = new BAlert( "About Touch...",
									   "Touch — a Tracker add-on\n"
									   "By Chris Herborth\n"
									   "\n"
									   "© 1999 Arcane Dragon Software\n"
									   "(arcanedragon@home.com)\n"
									   "\n"
									   "“Touches” the modification time of\n"
									   "the selected files.",
									   "Website", "OK", NULL,
									   B_WIDTH_AS_USUAL,
									   B_IDEA_ALERT );
			int32 button = info->Go();

			if( button == 0 ) {
				// TODO:
				// Display http://home.beoscentral.com/chrish/ads/
				//
				// BUG:
				// This opens two N+ windows, one is blank...
				int url_argc = 0;
				char *url_argv[3];
				url_argv[url_argc++] = const_cast<char *>( B_URL_HTTP );
				url_argv[url_argc++] = "http://home.beoscentral.com/chrish/ads/";
				url_argv[url_argc] = 0;

				(void)be_roster->Launch( B_URL_HTTP, url_argc, url_argv );
			}
		}
	}

	bool did_some_work;

	// With gcc you can initialize this here; with mwcc you can't.
	static const char *const _rcs_id;
};

const char *const TouchApplication::_rcs_id = "$Id: Touch.cpp,v 1.1.1.1 1999/06/28 18:03:22 chrish Exp $";

int main( void )
{
	TouchApplication app;
	app.PostMessage( B_QUIT_REQUESTED );	// So we exit right away.
	app.Run();

	return EXIT_SUCCESS;
}

// ----------------------------------------------------------------------
// Touch the files/folders selected in the Tracker window.

void process_refs( entry_ref dir_ref, BMessage *msg, void *data )
{
	// Keep compilers quiet.
	dir_ref = dir_ref;	// because we ignore it; the msg has the right data
	data = data;

	long count = 0;
	uint32 type = 0;
	
	msg->GetInfo( "refs", &type, &count );
	if( count == 0 ) return;

	if( data ) {
		TouchApplication *app = static_cast<TouchApplication *>( data );
		app->did_some_work = true;
	}

	for( int32 idx = 0; idx < count; idx++ ) {
		// Get the entry_ref, then touch it.
		entry_ref the_ref;

		status_t retval = msg->FindRef( "refs", idx, &the_ref );
		if( retval != B_NO_ERROR ) {
			// TODO: BAlert error - can't read refs!
			DoError( "Can't read refs from message!" );
		}

		BEntry ent( &the_ref );
		if( ent.InitCheck() == B_OK && ent.IsFile() ) {
			// touch me, touch me, I want to feel your body....
			(void)ent.SetModificationTime( time( NULL ) );
		}
	}
}
